#!/usr/bin/env python

import os
import sys
import distutils.spawn

boot_path=os.path.join(os.path.dirname(os.path.abspath(__file__)), "payload.dat")
exo_path =distutils.spawn.find_executable("exomizer")
__version__ = "1.0.0"


class Sid:
	def __init__(self,data):
		self.ID = data[:4]
		if self.ID != [80,83,73,68]:
			print("Not a valid sid file", file=sys.stderr)
			exit(1)
		self.version = data[5]
		self.dataoffset = data[7]
		self.loadaddress = 256 * data[8] + data[9] #if 0, then refer to prg load address
		
		if self.loadaddress == 0:
			self.loadaddress = data[self.dataoffset] + 256 * data[self.dataoffset + 1]
			self.dataoffset = self.dataoffset + 2
		self.initaddress = 	256 * data[10] + data[11]
		self.playaddress = 	256 * data[12] + data[13]
		self.songs = 		256 * data[14] + data[15]
		self.startsong = 	256 * data[16] + data [17]
		self.name =			bytearray(data[22:54]).decode("ASCII")
		self.author =		bytearray(data[54:86]).decode("ASCII")
		self.released =		bytearray(data[86:118]).decode("ASCII")
		self.data = data[self.dataoffset:]
		
if __name__ == "__main__":
	import argparse
	parser = argparse.ArgumentParser(prog='python '+os.path.basename(sys.argv[0]))
	parser.add_argument('-V', '--version', action='version', version='%(prog)s {version}'.format(version=__version__))
	parser.add_argument('-t', '--tune', dest = "tune", type = int, help = "subtune number")
	parser.add_argument('-s', '--slow', action = "store_true", help = "slows down the animation")
	parser.add_argument('infile',  help="(.sid file)")
	parser.add_argument('outfile', help="(.prg file)")
	
	args = parser.parse_args()
	sid = Sid(list(bytearray(open(args.infile,'rb').read())))
	tmpfile = args.outfile + "_"
	
	if sid.loadaddress < 0x801:
		print ("ERROR: sid starts before $0801", file=sys.stderr)
		exit(1)
	if args.tune !=None:
		if args.tune <= sid.songs and args.tune >=1:
			sid.startsong = args.tune
		else:
			print("ERROR: Not a valid sid subtune", file=sys.stderr)
			exit(1)
		
	if len(sid.data) + sid.loadaddress > 0x7bff:
		print("ERROR: Sid must fit in $0801-$07bff", file=sys.stderr)
		print("Please relocate this sid and try again", file=sys.stderr)
		exit(1)
	
	print("Sid info")
	
	print("Name:            %s"%sid.name)
	print("Author:          %s"%sid.author)
	print("Released:        %s"%sid.released)
	print("Load Address:    $%04x"%sid.loadaddress)
	print("Init Address:    $%04x"%sid.initaddress)
	print("play Address:    $%04x"%sid.playaddress)
	print("Number of tunes: %d"%sid.songs)
	print("Using tune:      %d\n"%sid.startsong)
	
	if exo_path == None:
		print("ERROR: can't find cruncher!", file=sys.stderr)
		print('Make sure Exomizer is installed. Please check out "INSTALLING" section of the file "readme.txt"')
		exit(1)

	prg = [0,] * 0x07ff + list(bytearray(open(boot_path,'rb').read()))
	prg[sid.loadaddress: sid.loadaddress + len(sid.data)] = sid.data
	prg[-6] = sid.startsong -1
	prg[-5] = 4 if args.slow else 3
	prg[-4], prg[-3] = sid.initaddress & 0xff, sid.initaddress >> 8
	prg[-2], prg[-1] = sid.playaddress & 0xff, sid.playaddress >> 8
	
	f = open(tmpfile, 'wb')
	f.write(bytearray(prg[0x07ff:]))
	f.close()
	
	#use exomizer to make the file executable
	print("Crunching file...")
	
	os.system("exomizer sfx $7c00 %s -n -q -o %s" %(tmpfile , args.outfile)) 
	os.remove(tmpfile)
	print("%s successfully created. Enjoy!" % args.outfile)
	
